/*************************************************************************
**  AH_MCP4922.h - Library for get out analog voltage          	        **
**  Created by A. Hinkel 2012-01-05					**
**  download from "http://www.alhin.de/arduino"  			**
**									**
**  Based on code from website: 					**
**  http://www.sinneb.net/2010/06/mcp4921-12bit-dac-voltage-controller/ **
**									**
**  Released into the public domain.  		                    	**
*************************************************************************/

#include "AH_MCP4922.h"

#define AI1 A1                                  //Analog input monitor 1
#define AI2 A2                                  //Analog input monitor 2

//define AnalogOutput (MOSI_pin, SCK_pin, CS_pin, DAC_x, GAIN) 
AH_MCP4922 AnalogOutput1(11,13,8,LOW,HIGH);    //define AnalogOutput1 for MEGA_board, select DAC_A, Gain=1x
AH_MCP4922 AnalogOutput2(11,13,8,HIGH,HIGH);    //define AnalogOutput2 for MEGA_board, select DAC_B, Gain=2x

/********************************
*            | MEGA  |  UNO     *
*  --------------------------   *
*  SCK_pin   |  52   |   13     *
*  MOSI_pin  |  51   |   11     *
*  CS_pin    |  53   |   12     *
*********************************
* DAC=LOW  => DAC_A select	*
* DAC=HIGH => DAC_B select	*
*				*
* GAIN=LOW  => 2x gain select	*
* GAIN=HIGH => 1x gain select	*
********************************/

//***************************************************************** 

#define SELPIN 9 //Selection Pin
#define DATAOUT 11//MOSI
#define DATAIN  12//MISO
#define SPICLOCK  13//Clock
int commandbits[] = {1,1,0,0,0}; //command bits (null,mode,channel,MSB/LSB)

void setup(){
  //set pin modes
  //disable device to start with
  digitalWrite(SELPIN,HIGH);
  digitalWrite(DATAOUT,LOW);
  digitalWrite(SPICLOCK,LOW);

  pinMode(SELPIN, OUTPUT);
  pinMode(DATAOUT, OUTPUT);
  pinMode(DATAIN, INPUT);
  pinMode(SPICLOCK, OUTPUT);
   
  Serial.begin(9600);           //Init serial interface 
  Serial.println("Ready");        
} 

//*****************************************************************
void loop()
{
  for (int i=0; i<513; i+=16)
  {
    Serial.println(i,DEC);
    AnalogOutput1.setValue(0);  // set voltage for AnalogOutput1 
    AnalogOutput2.setValue(i);  // set volateg for AnalogOutput2
    
    int a1val = analogRead(A1); // check the input voltage AI1 (pin A1)
    int a2val = analogRead(A2); // check the input voltage AI2 (pin A2)
    
    Serial.print("A1="); 
    Serial.print(a1val);
    Serial.print(" ; A2=");  
    Serial.println(a2val);

    Serial.print("ADx= "); 
    for(int j = 0; j < 4; j++)
    {
      int val = read_adc(j);
      Serial.print(j, DEC); 
      Serial.print(": "); 
      Serial.print(val,DEC);
      Serial.print("; "); 
    }
    Serial.println(""); 
    
    delay(2000);
  }
}


int read_adc(int channel)
{
  int tempbit = 0;
  int adcvalue = 0;
 
  // allow channel selection
  if(channel == 0)
  {
    commandbits[2] = 0;
    commandbits[3] = 0;
    commandbits[4] = 0;
  }
  else if(channel == 1)
  {
    commandbits[2] = 0;
    commandbits[3] = 0;
    commandbits[4] = 1;
  }
  else if(channel == 2)
  {
    commandbits[2] = 0;
    commandbits[3] = 1;
    commandbits[4] = 0;
  }
  else if(channel == 3)
  {
    commandbits[2] = 0;
    commandbits[3] = 1;
    commandbits[4] = 1;
  }
 
  digitalWrite(SELPIN,LOW); //Select adc

  // setup bits to be written
  for (int i=0; i<=4; i++)
  {
    if(commandbits[i]==1)
      digitalWrite(DATAOUT,HIGH);
    else
      digitalWrite(DATAOUT,LOW);

    // cycle clock
    digitalWrite(SPICLOCK,HIGH);
    delayMicroseconds(1);
    digitalWrite(SPICLOCK,LOW);
  }


  // one more clock cycle sampling time
  digitalWrite(SPICLOCK,HIGH);
  delayMicroseconds(1);
  digitalWrite(SPICLOCK,LOW);

  // read bits from adc
  for (int i=0; i<=12; i++)
  {
    tempbit = digitalRead(DATAIN);
   
    // ignores 1st null bit (always 0)
    adcvalue += tempbit * (1 << (12 - i));

    // cycle clock
    digitalWrite(SPICLOCK,HIGH);
    delayMicroseconds(1);
    digitalWrite(SPICLOCK,LOW);
  }

  digitalWrite(SELPIN, HIGH); //turn off device
  return adcvalue;
}

